@@ -35,10 +35,12 @@ and deleting resources.
     ListTasks = "/agent-execution/executions/history/{agent_id}"
     ListUserTasks = "/agent-execution/executions/history/user/{user_id}"
     GetTask = "/agent-execution/{task_id}/status"
+    GetTaskActivityLog = "/activity/{agent_id}/{task_id}"
     TaskCrud = "/agent-execution/{agent_or_task_id}"
     UpdateTask = "/agent-execution/{task_id}/update"
     ReportExternalTask = "/agent-execution/{agent_id}/report_task"
     ReportExecutionMetrics = "/agents-metrics/{agent_id}/execution"
+    PushExecutionEventToQueue = "/agent-execution/{task_id}/events/push"
     
     # Knowledge Bases Endpoints
     ListKnowledgeBases = "/knowledge_bases"
@@ -48,8 +50,24 @@ and deleting resources.
     KnowledgeBaseDocumentsCrud = "/knowledge_bases/{knowledge_base_id}"
     GetKnowledgeBaseDetails = "/knowledge_bases/{knowledge_base_id}/details"
     
+    # MCP Auth
+    GetUserMCPAuthToken = "/mcp_auth/{agent_id}/{user_id}/get_token"
+    
     # Tools
     GetOrInvokeToolById = "/tools/{tool_id}"
+    GetOrInvokeToolByUUID = "/tools/{connector_id}_{tool_id}"
+    ExecuteCodeInSandbox = "/tools/{task_id}/xp-code-executor"
+    
+    # HITL (Human-in-the-Loop)
+    HITLRequest = "/hitl/request"
+    HITLApprove = "/hitl/{task_id}/approve"
+    HITLReject = "/hitl/{task_id}/reject"
+    
+    # Metadata
+    GetOrgDefaultLLMExtraHeaders = "/metadata/default_llm_extra_headers"
+    
+    # Agentic context
+    GetOrSetAgenticContext = "/agentic_context/{agent_id}"
 
     def __str__(self) -> str:
         return str(self.value)
