@@ -12,7 +12,15 @@ class Framework(str, Enum):
     """
 
     Agno = "agno"
+    GoogleADK = "google-adk"
+    LangChain = "langchain"
+    OpenAIAgents = "open-ai-agents"
+    Strands = "strands-agents"
 
+class AgnoToolCallsCompressionSettings(BaseModel):
+    enabled: Optional[bool] = False
+    threshold: Optional[int] = 3
+    instructions: Optional[str] = ""
 
 class AgnoSettings(BaseModel):
     """
@@ -22,7 +30,7 @@ class AgnoSettings(BaseModel):
         session_storage (Optional[bool]): If True, enables session-level storage. Default is True.
         user_memories (Optional[bool]): If True, enables memory of user interactions. Default is True.
         session_summaries (Optional[bool]): If True, enables generation of session summaries. Default is False.
-        num_history_runs (Optional[int]): Number of historical runs to retain or consider. Default is 3.
+        num_history_runs (Optional[int]): Number of historical runs to retain or consider. Default is 10.
         tool_call_limit (Optional[int]): Max tool calls per run.
         coordinate_mode (Optional[bool]): If True, The agent will be loaded as a Team. Default is False.
         pii_detection_enabled (Optional[bool]): If True, enables PII detection guardrail on agent input. Default is False.
@@ -34,9 +42,15 @@ class AgnoSettings(BaseModel):
     """
 
     session_storage: Optional[bool] = True
-    user_memories: Optional[bool] = True
+    
+    user_memories: Optional[bool] = False
+    agentic_memory: Optional[bool] = False
+    
+    agent_memories: Optional[bool] = False
+    agentic_culture: Optional[bool] = False
+    
     session_summaries: Optional[bool] = False
-    num_history_runs: Optional[int] = 3
+    num_history_runs: Optional[int] = 10
     max_tool_calls_from_history: Optional[int] = 0
     tool_call_limit: Optional[int] = None
     coordinate_mode: Optional[bool] = False
@@ -46,3 +60,5 @@ class AgnoSettings(BaseModel):
     openai_moderation_enabled: Optional[bool] = False
     openai_moderation_categories: Optional[List[str]] = None
     reasoning_tools_enabled: Optional[bool] = False
+    
+    tool_calls_compression: Optional[AgnoToolCallsCompressionSettings] = None
