@@ -10,6 +10,7 @@ from enum import Enum
 from typing import Dict, List, Literal, Optional, Type
 from pydantic import BaseModel, computed_field
 
+from xpander_sdk.models.orchestrations import DuplicationPreventionSettings, OrchestrationIterativeStrategy, OrchestrationRetryStrategy, OrchestrationStopStrategy
 from xpander_sdk.models.shared import XPanderSharedModel
 from xpander_sdk.modules.tools_repository.models.mcp import MCPServerDetails
 
@@ -72,7 +73,7 @@ class AgentInstructions(BaseModel):
     
     @computed_field
     @property
-    def instructions(self) -> List[str]:
+    def instructions(self) -> str:
         """
         Get the role-specific instructions for the agent.
         
@@ -87,6 +88,27 @@ class AgentInstructions(BaseModel):
             {self.goal_str}
         </goals>
         """
+    
+    @computed_field
+    @property
+    def full(self) -> str:
+        """
+        Get the role-specific instructions for the agent.
+        
+        Returns:
+            List[str]: List of role instructions.
+        """
+        return f"""
+        <description>
+            {self.description}
+        </description>
+        <instructions>
+            {self.role}
+        </instructions>
+        <goals>
+            {self.goal_str}
+        </goals>
+        """
 
     @computed_field
     @property
@@ -362,6 +384,33 @@ class AgentGraphItem(BaseModel):
     llm_settings: Optional[List[AgentGraphItemLLMSettings]] = []
     is_first: Optional[bool] = False
 
+class LLMReasoningEffort(str, Enum):
+    Low = "low"
+    Medium = "medium"
+    High = "high"
+
+class AIAgentConnectivityDetailsA2AAuthType(str, Enum):
+    NoAuth = "none"
+    ApiKey = "api_key"
+    Basic = "basic"
+    Bearer = "bearer"
+
+class AIAgentConnectivityDetailsBase(XPanderSharedModel):
+    custom_headers: Optional[Dict[str, str]] = {}
+    auth_type: Optional[AIAgentConnectivityDetailsA2AAuthType] = AIAgentConnectivityDetailsA2AAuthType.NoAuth
+    api_key_header_name: Optional[str] = "X-API-Key"
+    auth_value: Optional[str] = None
+
+class AIAgentConnectivityDetailsA2A(AIAgentConnectivityDetailsBase):
+    agent_url: str
+
+class AIAgentConnectivityDetailsCurl(AIAgentConnectivityDetailsBase):
+    curl_command: str
+    invoke_url: Optional[str] = None # post analysis (auto populate by llm)
+    method: Optional[str] = "POST" # post analysis (auto populate by llm)
+    prompt_field_name: Optional[str] = "" # post analysis (auto populate by llm) - CAN BE SET BY THE USER
+    files_field_name: Optional[str] = "" # post analysis (auto populate by llm) - CAN BE SET BY THE USER
+    task_id_field_name: Optional[str] = "" # post analysis (auto populate by llm) - CAN BE SET BY THE USER
 
 class AgentType(str, Enum):
     """
@@ -370,10 +419,16 @@ class AgentType(str, Enum):
     Values:
         Manager: Agent that manages and coordinates other agents.
         Regular: Standard agent for individual task execution.
+        A2A: Agent that is used via A2A protocol.
+        Curl: Custom Agent that is used via curl.
+        Orchestration: marks the agent as an Orchestration object.
     """
 
     Manager = "manager"
     Regular = "regular"
+    A2A = "a2a"
+    Curl = "curl"
+    Orchestration = "orchestration"
 
 
 @dataclass
@@ -419,7 +474,38 @@ class AgentOutput(BaseModel):
     is_markdown: Optional[bool] = False
     use_json_mode: Optional[bool] = False
 
-class LLMCredentials(XPanderSharedModel):
-    name: str
-    description: Optional[str] = None
-    value: str
\ No newline at end of file
+class TaskLevelStrategies(XPanderSharedModel):
+    """
+    Configuration object for task-level execution strategies.
+
+    This model groups optional strategy configurations that control how a task is
+    executed and managed over time, including retries, iterative execution,
+    stopping conditions, and daily run limits.
+
+    Attributes:
+        retry_strategy:
+            Optional retry policy configuration that defines how the task should
+            behave when execution fails (e.g., max attempts, backoff rules).
+
+        iterative_strategy:
+            Optional iterative execution configuration for tasks that may run in
+            repeated cycles/steps until completion or a stop condition is met.
+
+        stop_strategy:
+            Optional stopping policy configuration that defines when the task
+            should stop running (e.g., timeout, max iterations, success criteria).
+
+        max_runs_per_day:
+            Optional limit on how many times the task is allowed to run within a
+            24-hour period. If not set, no explicit daily limit is enforced.
+        
+        agentic_context_enabled:
+            if agentic memory is enabled and accesible to the executor.
+    """
+
+    retry_strategy: Optional[OrchestrationRetryStrategy] = None
+    iterative_strategy: Optional[OrchestrationIterativeStrategy] = None
+    stop_strategy: Optional[OrchestrationStopStrategy] = None
+    max_runs_per_day: Optional[int] = None
+    agentic_context_enabled: Optional[bool] = False
+    duplication_prevention: Optional[DuplicationPreventionSettings] = None
