@@ -8,16 +8,19 @@ agents in the xpander.ai Backend-as-a-Service platform.
 import asyncio
 from datetime import datetime
 import heapq
-from typing import Any, Callable, Dict, List, Optional, Type, TypeVar
+import re
+from typing import Any, Callable, Dict, List, Optional, Type, TypeVar, Union
 from httpx import HTTPStatusError
 from loguru import logger
 from pydantic import ConfigDict, computed_field
-
+from strands import tool as strands_tool
 from xpander_sdk.consts.api_routes import APIRoute
 from xpander_sdk.core.xpander_api_client import APIClient
 from xpander_sdk.exceptions.module_exception import ModuleException
 from xpander_sdk.models.configuration import Configuration
 from xpander_sdk.models.frameworks import AgnoSettings, Framework
+from xpander_sdk.models.notifications import NotificationSettings
+from xpander_sdk.models.orchestrations import OrchestrationNode
 from xpander_sdk.modules.agents.utils.generic import get_db_schema_name
 from xpander_sdk.modules.knowledge_bases.models.knowledge_bases import (
     KnowledgeBaseSearchResult,
@@ -26,6 +29,8 @@ from xpander_sdk.modules.tools_repository.models.mcp import MCPServerDetails
 from xpander_sdk.models.shared import LLMModelT, OutputFormat, ThinkMode, XPanderSharedModel
 from xpander_sdk.models.user import User
 from xpander_sdk.modules.agents.models.agent import (
+    AIAgentConnectivityDetailsA2A,
+    AIAgentConnectivityDetailsCurl,
     AgentAccessScope,
     AgentDeploymentType,
     AgentGraphItem,
@@ -36,8 +41,10 @@ from xpander_sdk.modules.agents.models.agent import (
     AgentStatus,
     AgentType,
     DatabaseConnectionString,
-    LLMCredentials,
+    LLMReasoningEffort,
+    TaskLevelStrategies,
 )
+from xpander_sdk.models.generic import LLMCredentials
 from xpander_sdk.modules.agents.models.knowledge_bases import AgentKnowledgeBase
 from xpander_sdk.modules.knowledge_bases.knowledge_bases_module import KnowledgeBases
 from xpander_sdk.modules.knowledge_bases.sub_modules.knowledge_base import KnowledgeBase
@@ -50,6 +57,7 @@ from xpander_sdk.modules.tools_repository.sub_modules.tool import Tool
 from xpander_sdk.modules.tools_repository.tools_repository_module import ToolsRepository
 from xpander_sdk.modules.tools_repository.utils.schemas import build_model_from_schema
 from xpander_sdk.utils.event_loop import run_sync
+from xpander_sdk.utils.tools import get_openai_agents_sdk_tools
 
 
 class AgentGraph(XPanderSharedModel):
@@ -147,15 +155,28 @@ class Agent(XPanderSharedModel):
             using_nemo: Optional[bool]
             model_provider: str
             model_name: str
+            llm_reasoning_effort: Optional[LLMReasoningEffort] = LLMReasoningEffort.Medium
+            deep_planning: Optional[bool] = False
+            enforce_deep_planning: Optional[bool] = False
             llm_api_base: Optional[str]
+            llm_extra_headers: Optional[Dict[str,str]]
             webhook_url: Optional[str]
             created_at: Optional[datetime]
             type: Optional[AgentType]
+            connectivity_details: Optional[Union[AIAgentConnectivityDetailsA2A,AIAgentConnectivityDetailsCurl, Dict]] = {}
             output_format: Optional[OutputFormat]
             output_schema: Optional[Dict]
             llm_credentials: Optional[LLMCredentials]
             expected_output: Optional[str]
             agno_settings: Optional[AgnoSettings]
+            orchestration_nodes: Optional[List[OrchestrationNode]] = []
+            notification_settings: Optional[NotificationSettings] = {}
+            task_level_strategies: Optional[TaskLevelStrategies] = None
+            use_oidc_pre_auth: Optional[bool] = False
+            pre_auth_audiences: Optional[List[str]] = []
+            use_oidc_pre_auth_token_for_llm: Optional[bool] = False
+            oidc_pre_auth_token_mcp_audience: Optional[bool] = False
+            oidc_pre_auth_token_llm_audience: Optional[str] = None
 
         Example:
             >>> agent = Agent(id="agent123", name="Example Agent")
@@ -171,6 +192,7 @@ class Agent(XPanderSharedModel):
     environment_id: str = None
     tools: Optional[ToolsRepository] = None
     icon: Optional[str] = "🚀"
+    connectivity_details: Optional[Union[AIAgentConnectivityDetailsA2A,AIAgentConnectivityDetailsCurl, Dict]] = {}
     deployment_type: Optional[AgentDeploymentType] = AgentDeploymentType.Serverless
     source_nodes: Optional[List[AgentSourceNode]] = []
     access_scope: Optional[AgentAccessScope] = AgentAccessScope.Organizational
@@ -186,7 +208,11 @@ class Agent(XPanderSharedModel):
     using_nemo: Optional[bool] = False
     model_provider: str
     model_name: str
+    llm_reasoning_effort: Optional[LLMReasoningEffort] = LLMReasoningEffort.Medium
+    deep_planning: Optional[bool] = False
+    enforce_deep_planning: Optional[bool] = False
     llm_api_base: Optional[str] = None
+    llm_extra_headers: Optional[Dict[str,str]] = {}
     webhook_url: Optional[str] = None
     created_at: Optional[datetime] = None
     type: Optional[AgentType] = None
@@ -196,6 +222,14 @@ class Agent(XPanderSharedModel):
     llm_credentials: Optional[LLMCredentials] = None
     expected_output: Optional[str] = ""
     agno_settings: Optional[AgnoSettings] = AgnoSettings()
+    orchestration_nodes: Optional[List[OrchestrationNode]] = []
+    notification_settings: Optional[NotificationSettings] = {}
+    task_level_strategies: Optional[TaskLevelStrategies] = None
+    use_oidc_pre_auth: Optional[bool] = False
+    pre_auth_audiences: Optional[List[str]] = []
+    use_oidc_pre_auth_token_for_llm: Optional[bool] = False
+    oidc_pre_auth_token_llm_audience: Optional[str] = None
+    oidc_pre_auth_token_mcp_audience: Optional[str] = None
 
     _connection_string: Optional[DatabaseConnectionString] = None
 
@@ -233,7 +267,7 @@ class Agent(XPanderSharedModel):
             List[MCPServerDetails]: Details of MCP servers configured for this agent.
         """
         return [
-            gi.settings.mcp_settings
+            MCPServerDetails(**gi.settings.mcp_settings) if isinstance(gi.settings.mcp_settings, dict) else gi.settings.mcp_settings
             for gi in self.graph.items
             if gi.type == AgentGraphItemType.MCP
         ]
@@ -389,11 +423,17 @@ class Agent(XPanderSharedModel):
         output_schema: Optional[Dict] = None,
         events_streaming: Optional[bool] = False,
         additional_context: Optional[str] = None,
+        instructions_override: Optional[str] = None,
+        test_run_node_id: Optional[str] = None,
+        user_oidc_token: Optional[str] = None,
         expected_output: Optional[str] = None,
         mcp_servers: Optional[List[MCPServerDetails]] = [],
         triggering_agent_id: Optional[str] = None,
         title: Optional[str] = None,
-        think_mode: Optional[ThinkMode] = ThinkMode.Default
+        think_mode: Optional[ThinkMode] = ThinkMode.Default,
+        disable_attachment_injection: Optional[bool] = False,
+        return_metrics: Optional[bool] = False,
+        user_tokens: Optional[Dict] = None
     ) -> Task:
         """
         Asynchronously create a new task and link it to this agent.
@@ -412,11 +452,17 @@ class Agent(XPanderSharedModel):
             output_schema (Optional[Dict]): Schema defining structure of output.
             events_streaming (Optional[bool]): Flag idicating for events are required for this task.
             additional_context (Optional[str]): Additional context to be passed to the agent.
+            instructions_override (Optional[str]): Additional instructions to be appended to the agent's system prompt.
+            test_run_node_id (Optional[str]): Internal test purpose workflow node id to execute.
+            user_oidc_token (Optional[str]): OIDC Token to be used for pre-auth.
             expected_output (Optional[str]): Expected output of the execution.
             mcp_servers (Optional[List[MCPServerDetails]]): Optional list of mcp servers to use.
             triggering_agent_id (Optional[str]): Optional triggering agent id.
             title (Optional[str]): Optional task title.
             think_mode (Optional[ThinkMode]): Optional task think mode, defaults to "default".
+            disable_attachment_injection (Optional[bool]): Optional selection if to disable attachment injection to the context window.
+            return_metrics (Optional[bool]): Optional selection if to return metrics report. Available only for Workflow -> Agent invocation
+            user_tokens: Optional[Dict]: User tokens to be passed and injected for MCP Auth
 
         Returns:
             Task: Created Task object linked to this agent.
@@ -445,11 +491,17 @@ class Agent(XPanderSharedModel):
                     "run_locally": run_locally,
                     "events_streaming": events_streaming,
                     "additional_context": additional_context,
+                    "instructions_override": instructions_override,
+                    "test_run_node_id": test_run_node_id,
+                    "user_oidc_token": user_oidc_token,
                     "expected_output": expected_output,
                     "mcp_servers": [server.model_dump() for server in mcp_servers],
                     "triggering_agent_id": triggering_agent_id,
                     "title": title,
                     "think_mode": think_mode.value,
+                    "disable_attachment_injection": disable_attachment_injection,
+                    "user_tokens": user_tokens,
+                    "return_metrics": return_metrics
                 },
             )
             return Task(**created_task, configuration=self.configuration)
@@ -586,7 +638,8 @@ class Agent(XPanderSharedModel):
             ImportError: If required dependencies are missing.
             ValueError: If the connection string for storage is invalid.
         """
-        if self.framework != Framework.Agno:
+        framework = Framework.Agno # will be removed
+        if self.framework != framework:
             raise NotImplementedError(
                 f"Storage for framework '{self.framework}' is not supported."
             )
@@ -611,12 +664,12 @@ class Agent(XPanderSharedModel):
         schema = get_db_schema_name(agent_id=self.id)
         
         client_type = AsyncPostgresDb if async_db else PostgresDb
-
+        
         return client_type(
             db_schema=schema,
             db_url=connection_string.connection_uri.uri.replace("postgresql", "postgresql+psycopg"+("_async" if async_db else "")),
         )
-
+    
     def get_db(self) -> Any:
         """
         Synchronously retrieve the db for this agent.
@@ -734,9 +787,9 @@ class Agent(XPanderSharedModel):
         Example:
             >>> sessions = await agent.aget_user_sessions(user_id="user_123")
         """
-        db = await self.aget_db()
+        db = await self.aget_db(async_db=True)
         from agno.db import SessionType
-        sessions = await asyncio.to_thread(db.get_sessions, user_id=user_id, limit=50, session_type = SessionType.TEAM if self.is_a_team else SessionType.AGENT)
+        sessions = await db.get_sessions(user_id=user_id, limit=50, session_type = SessionType.TEAM if self.is_a_team else SessionType.AGENT)
         return sessions
     
     def get_user_sessions(self, user_id: str):
@@ -780,10 +833,9 @@ class Agent(XPanderSharedModel):
         Example:
             >>> session = await agent.aget_session(session_id="sess_456")
         """
-        db = await self.aget_db()
+        db = await self.aget_db(async_db=True)
         from agno.db import SessionType
-        session = await asyncio.to_thread(db.get_session, session_id=session_id, session_type = SessionType.TEAM if self.is_a_team else SessionType.AGENT)
-        return await session
+        return await db.get_session(session_id=session_id, session_type = SessionType.TEAM if self.is_a_team else SessionType.AGENT)
 
     def get_session(self, session_id: str):
         """
@@ -824,8 +876,8 @@ class Agent(XPanderSharedModel):
         Example:
             >>> await agent.adelete_session(session_id="sess_456")
         """
-        db = await self.aget_db(async_db=False)
-        await asyncio.to_thread(db.delete_session, session_id=session_id)
+        db = await self.aget_db(async_db=True)
+        await db.delete_session(session_id=session_id)
     
     def delete_session(self, session_id: str):
         """
@@ -880,3 +932,61 @@ class Agent(XPanderSharedModel):
 
         if not any(kb.id == knowledge_base_id for kb in self.knowledge_bases):
             self.knowledge_bases.append(AgentKnowledgeBase(id=knowledge_base_id))
+
+    @computed_field
+    @property
+    def strands_tools(self) -> List[Any]:
+        tools = []
+        for _tool in self.tools.list:
+            def make_tool(_tool_def: Tool):
+                async def invoke(payload: dict):
+                    return await _tool_def.ainvoke(
+                        task_id=self.configuration.state.task.id
+                            if self.configuration.state.task
+                            else None,
+                        agent_id=self.id,
+                        agent_version=self.version,
+                        payload=payload,
+                        configuration=_tool.configuration
+                    )
+                return invoke
+            
+            tools.append(
+                strands_tool(
+                    func=make_tool(_tool),
+                    name=_tool.id,
+                    description=_tool.description,
+                    inputSchema={
+                        "json": {
+                            "type": "object",
+                            "properties":{
+                                "payload": _tool.parameters
+                            },
+                            "required": ["payload"]
+                        }
+                    }
+                )
+            )
+        
+        return tools
+    
+    @computed_field
+    @property
+    def sanitized_name(self) -> str:
+        # Replace invalid characters with underscores
+        sanitized = re.sub(r'[^A-Za-z0-9_]', '_', self.name)
+
+        # If the first character is invalid, prefix with underscore
+        if not re.match(r'[A-Za-z_]', sanitized):
+            sanitized = '_' + sanitized
+
+        # Prevent empty result
+        if not sanitized:
+            sanitized = '_agent'
+
+        return sanitized
+    
+    @computed_field
+    @property
+    def openai_agents_sdk_tools(self) -> List[Any]:
+        return get_openai_agents_sdk_tools(self)
\ No newline at end of file
