@@ -9,6 +9,7 @@ from __future__ import annotations
 
 import asyncio
 import json
+import json as py_json
 import os
 import signal
 import sys
@@ -19,10 +20,12 @@ from typing import Any, Awaitable, Callable, Optional, Set, Union, List
 import httpx
 from httpx_sse import aconnect_sse
 from loguru import logger
+from pydantic import BaseModel
 
 from xpander_sdk.core.module_base import ModuleBase
 from xpander_sdk.exceptions.module_exception import ModuleException
 from xpander_sdk.models.configuration import Configuration
+from xpander_sdk.models.shared import OutputFormat
 from xpander_sdk.modules.agents.models.agent import SourceNodeType
 from xpander_sdk.modules.tasks.tasks_module import Tasks
 
@@ -34,6 +37,7 @@ from .models.events import (
     WorkerEnvironmentConflict,
     WorkerFinishedEvent,
     WorkerHeartbeat,
+    WorkerCapacityUpdateEvent,
 )
 from ..tasks.sub_modules.task import Task
 from ..tasks.models.task import AgentExecutionStatus, LocalTaskTest
@@ -78,6 +82,7 @@ class Events(ModuleBase):
 
     worker: Optional[DeployedAsset] = None
     test_task: Optional[LocalTaskTest] = None
+    _heartbeat_task: Optional[asyncio.Task] = None
     
     # Class-level registries for boot and shutdown handlers
     _boot_handlers: List[BootHandler] = []
@@ -86,7 +91,7 @@ class Events(ModuleBase):
     def __init__(
         self,
         configuration: Optional[Configuration] = None,
-        max_sync_workers: Optional[int] = 4,
+        max_sync_workers: Optional[int] = 6,
         max_retries: Optional[int] = _MAX_RETRIES,
     ):
         """
@@ -96,7 +101,7 @@ class Events(ModuleBase):
 
         Args:
             configuration (Optional[Configuration]): SDK configuration with credentials and endpoints. Defaults to environment configuration.
-            max_sync_workers (Optional[int]): Maximum number of synchronous worker threads. Defaults to 4.
+            max_sync_workers (Optional[int]): Maximum number of synchronous worker threads. Defaults to 6.
             max_retries (Optional[int]): Maximum retry attempts for network calls. Defaults to 5.
 
         Raises:
@@ -106,7 +111,7 @@ class Events(ModuleBase):
         configure_git_credentials()
 
         self.is_xpander_cloud = getenv("IS_XPANDER_CLOUD", "false") == "true"
-        self.agent_id = getenv("XPANDER_AGENT_ID", None)
+        self.agent_id = self.configuration.agent_id or getenv("XPANDER_AGENT_ID", None)
 
         if not self.agent_id:
             raise ModuleException(
@@ -123,6 +128,7 @@ class Events(ModuleBase):
             )
 
         self.max_retries = max_retries
+        self.max_sync_workers = max_sync_workers
 
         # Internal resources
         self._pool: ThreadPoolExecutor = ThreadPoolExecutor(
@@ -130,6 +136,7 @@ class Events(ModuleBase):
             thread_name_prefix="xpander-handler",
         )
         self._bg: Set[asyncio.Task] = set()
+        self._execution_semaphore: Optional[asyncio.Semaphore] = None
 
         logger.debug(
             f"Events initialised (base_url={self.configuration.base_url}, "
@@ -155,6 +162,9 @@ class Events(ModuleBase):
         # Execute boot handlers first, before any event listeners are set up
         await self._execute_boot_handlers()
         
+        # Initialize semaphore for capacity tracking
+        self._execution_semaphore = asyncio.Semaphore(self.max_sync_workers)
+        
         loop = asyncio.get_running_loop()
         for sig in (signal.SIGINT, signal.SIGTERM):
             loop.add_signal_handler(
@@ -256,6 +266,22 @@ class Events(ModuleBase):
         assert last_exc is not None
         raise last_exc  # for static checkers
 
+    async def _notify_capacity_status(self, worker_id: str, is_busy: bool) -> None:
+        """
+        Notify the backend of the worker's capacity status.
+
+        Args:
+            worker_id (str): The unique identifier of the worker.
+            is_busy (bool): Whether the worker is at max capacity.
+        """
+        url = f"{get_events_base(configuration=self.configuration)}/{worker_id}?type=worker&agent_id={self.agent_id}"
+        await self._request_with_retries(
+            "POST",
+            url,
+            headers=get_events_headers(configuration=self.configuration),
+            json=WorkerCapacityUpdateEvent(data={"is_busy": is_busy}).model_dump_safe(),
+        )
+
     async def _release_worker(self, worker_id: str) -> None:
         """
         Release the worker resource after task execution completion.
@@ -322,11 +348,37 @@ class Events(ModuleBase):
                 await asyncio.sleep(backoff_delay(attempt))
                 attempt += 1
 
+    async def _handle_task_with_semaphore(
+        self,
+        agent_worker: DeployedAsset,
+        task: Task,
+        on_execution_request: ExecutionRequestHandler,
+    ) -> None:
+        """
+        Wrapper that releases semaphore after task execution.
+        Semaphore is already acquired before calling this method.
+        """
+        try:
+            await self.handle_task_execution_request(
+                agent_worker, task, on_execution_request
+            )
+        finally:
+            # Release execution slot
+            self._execution_semaphore.release()
+            
+            # Check if we now have available capacity and notify backend
+            if self._execution_semaphore._value > 0:
+                try:
+                    await self._notify_capacity_status(agent_worker.id, is_busy=False)
+                except Exception as e:
+                    logger.warning(f"Failed to notify available status: {e}")
+    
     async def handle_task_execution_request(
         self,
         agent_worker: DeployedAsset,
         task: Task,
         on_execution_request: ExecutionRequestHandler,
+        retry_count: Optional[int] = 0,
     ) -> None:
         """
         Handle an incoming task execution request.
@@ -335,6 +387,7 @@ class Events(ModuleBase):
             agent_worker (DeployedAsset): The deployed asset (agent) to handle the task.
             task (Task): The task object containing execution details.
             on_execution_request (ExecutionRequestHandler): The handler function to process the task.
+            retry_count (Optional[int]): Current retry attempt count. Defaults to 0.
         """
         error = None
         try:
@@ -348,22 +401,50 @@ class Events(ModuleBase):
                     on_execution_request,
                     task,
                 )
+            
+            # Check if plan is complete, retry if not
+            plan_following_status = await task.aget_plan_following_status()
+            if not plan_following_status.can_finish:
+                # Check if we've exceeded max retries
+                if retry_count >= 50:  # 0, 1, 2 = 50 total attempts
+                    logger.warning(f"Failed to complete plan after {retry_count + 1} attempts. Remaining incomplete tasks.")
+                    return
+                
+                # Recursively call with incremented retry count
+                logger.info(f"Plan not complete, retrying (attempt {retry_count + 2})")
+                await self.handle_task_execution_request(
+                    agent_worker,
+                    task,
+                    on_execution_request,
+                    retry_count=retry_count + 1
+                )
+                return
+            
         except Exception as e:
             logger.exception(f"Execution handler failed - {str(e)}")
             error = str(e)
         finally:
             task_used_tokens = task.tokens
             task_used_tools = task.used_tools
-            await self._release_worker(agent_worker.id)
 
             if error:
                 task.result = error
-                task.status = AgentExecutionStatus.Failed
+                task.status = AgentExecutionStatus.Error
             elif (
                 task.status == AgentExecutionStatus.Executing
             ):  # let the handler set the status, if not set - mark as completed
                 task.status = AgentExecutionStatus.Completed
 
+            # in case of structured output, return as stringified json
+            try:
+                if task.output_format == OutputFormat.Json:
+                    if isinstance(task.result, BaseModel):
+                        task.result = task.result.model_dump_json()
+                    if isinstance(task.result, dict) or isinstance(task.result, list):
+                        task.result = py_json.dumps(task.result)
+            except Exception:
+                pass
+            
             await task.asave()
             task.tokens = task_used_tokens
             task.used_tools = task_used_tools
@@ -466,13 +547,29 @@ class Events(ModuleBase):
                     )
                 )
 
-                self.track(asyncio.create_task(self.heartbeat_loop(agent_worker.id)))
+                # Cancel previous heartbeat task if it exists and start a new one
+                if self._heartbeat_task and not self._heartbeat_task.done():
+                    logger.debug(f"Canceling previous heartbeat task for worker {agent_worker.id}")
+                    self._heartbeat_task.cancel()
+                self._heartbeat_task = asyncio.create_task(self.heartbeat_loop(agent_worker.id))
+                self.track(self._heartbeat_task)
 
             elif event.event == EventType.AgentExecution:
                 task = Task(**json.loads(event.data), configuration=self.configuration)
+                
+                # Acquire execution slot immediately (blocks here if at max capacity)
+                await self._execution_semaphore.acquire()
+                
+                # Check if we're now at max capacity and notify backend
+                if self._execution_semaphore._value == 0:
+                    try:
+                        await self._notify_capacity_status(agent_worker.id, is_busy=True)
+                    except Exception as e:
+                        logger.warning(f"Failed to notify busy status: {e}")
+                
                 self.track(
                     asyncio.create_task(
-                        self.handle_task_execution_request(
+                        self._handle_task_with_semaphore(
                             agent_worker, task, on_execution_request
                         )
                     )
