@@ -14,6 +14,7 @@ class EventType(str, Enum):
     AgentExecution = "agent-execution"
     WorkerHeartbeat = "worker-heartbeat"
     WorkerFinished = "worker-finished"
+    WorkerCapacityUpdate = "worker-capacity-update"
     EnvironmentConflict = "worker-environment-conflict"
 
 
@@ -52,6 +53,11 @@ class WorkerExecutionRequest(EventMessageBase):
     data: Task
 
 
+class WorkerCapacityUpdateEvent(EventMessageBase):
+    event: EventType = EventType.WorkerCapacityUpdate
+    data: dict = {"is_busy": False}
+
+
 class WorkerEnvironmentConflict(EventMessageBase):
     event: EventType = EventType.EnvironmentConflict
     error: str
