@@ -234,11 +234,17 @@ class Tasks(ModuleBase):
         output_schema: Optional[Dict] = None,
         events_streaming: Optional[bool] = False,
         additional_context: Optional[str] = None,
+        instructions_override: Optional[str] = None,
+        test_run_node_id: Optional[str] = None,
+        user_oidc_token: Optional[str] = None,
         expected_output: Optional[str] = None,
         mcp_servers: Optional[List[MCPServerDetails]] = [],
         triggering_agent_id: Optional[str] = None,
         title: Optional[str] = None,
-        think_mode: Optional[ThinkMode] = ThinkMode.Default
+        think_mode: Optional[ThinkMode] = ThinkMode.Default,
+        disable_attachment_injection: Optional[bool] = False,
+        return_metrics: Optional[bool] = False,
+        user_tokens: Optional[Dict] = None
     ) -> Task:
         """
         Asynchronously create a new task for a specific agent.
@@ -261,11 +267,17 @@ class Tasks(ModuleBase):
             output_schema (Optional[Dict]): Schema defining the expected output structure.
             events_streaming (Optional[bool]): Flag idicating for events are required for this task.
             additional_context (Optional[str]): Additional context to be passed to the agent.
+            instructions_override (Optional[str]): Additional instructions to be appended to the agent's system prompt.
+            test_run_node_id (Optional[str]): Internal test purpose workflow node id to execute.
+            user_oidc_token (Optional[str]): OIDC Token to be used for pre-auth
             expected_output (Optional[str]): Expected output of the execution.
             mcp_servers (Optional[List[MCPServerDetails]]): Optional list of mcp servers to use.
             triggering_agent_id (Optional[str]): Optional triggering agent id.
             title (Optional[str]): Optional task title.
             think_mode (Optional[ThinkMode]): Optional task think mode, defaults to "default".
+            disable_attachment_injection (Optional[bool]): Optional selection if to disable attachment injection to the context window.
+            return_metrics (Optional[bool]): Optional selection if to return metrics report. Available only for Workflow -> Agent invocation
+            user_tokens: Optional[Dict]: User tokens to be passed and injected for MCP Auth
 
         Returns:
             Task: Newly created task object containing all initial configuration data.
@@ -305,11 +317,17 @@ class Tasks(ModuleBase):
                     "run_locally": run_locally,
                     "events_streaming": events_streaming,
                     "additional_context": additional_context,
+                    "instructions_override": instructions_override,
+                    "test_run_node_id": test_run_node_id,
+                    "user_oidc_token": user_oidc_token,
                     "expected_output": expected_output,
                     "mcp_servers": [server.model_dump() for server in mcp_servers],
                     "triggering_agent_id": triggering_agent_id,
                     "title": title,
                     "think_mode": think_mode.value,
+                    "disable_attachment_injection": disable_attachment_injection,
+                    "user_tokens": user_tokens,
+                    "return_metrics": return_metrics
                 },
             )
             return Task(**created_task, configuration=self.configuration)
